;;; Directory Local Variables            -*- no-byte-compile: t -*-
;;; For more information see (info "(emacs) Directory Variables")

((cask-mode . ((indent-tabs-mode . nil)))
 (emacs-lisp-mode . ((sentence-end-double-space . t)
                     (indent-tabs-mode . nil)
                     (mode . bug-reference-prog))))
